Package: FRESA.CAD
Type: Package
Title: Feature Selection Algorithms for Computer Aided Diagnosis
Version: 3.1.0
Date: 2018-10-30
Author: Jose Gerardo Tamez-Pena, Antonio Martinez-Torteya and Israel Alanis
Maintainer: Jose Gerardo Tamez-Pena <jose.tamezpena@tec.mx>
Description: Contains a set of utilities for building and testing statistical models (linear, logistic,ordinal or COX) for Computer Aided Diagnosis/Prognosis applications. Utilities include data adjustment, univariate analysis, model building, model-validation, longitudinal analysis, reporting and visualization.
License: LGPL (>= 2)
Depends: Rcpp (>= 0.10.0),stringr,miscTools,Hmisc,pROC
LinkingTo: Rcpp, RcppArmadillo
Suggests:
        nlme,rpart,gplots,RColorBrewer,class,cvTools,glmnet,randomForest,survival,e1071,MASS,naivebayes,mRMRe,epiR,DescTools,irr
Packaged: 2018-11-27 03:53:27 UTC; Jose Tamez
NeedsCompilation: yes
Repository: CRAN
Date/Publication: 2018-11-27 08:50:03 UTC
